﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace EnemyPlacementTester
{
    [Serializable]
    public class CDiagram
    {
        #region fields and their properties
        private ArrayList items;

        public ArrayList Items
        {
            get { return items; }
        }

        private int currentSelection;

        public int CurrentSelection
        {
            get { return currentSelection; }
            set { currentSelection = value; }
        }
        #endregion

        #region constructor
        public CDiagram()
        {
            items = new ArrayList();
            currentSelection = -1;
        }
        #endregion

        #region methods

        // checks to see if the user is clicking within the area
        public bool clickingWithin(Point mouseClick)
        {
            // prevents the index being out of range
            if (isValidSelection())
            {
                return (((IDiagramItem)items[currentSelection]).containsPoint(mouseClick));
            }
            else
            {
                return false;
            }
        }

        // draws the items
        public void drawItems(Graphics g, bool showVision, bool showRepulsion)
        {
            int count = 0;

            // loops through every item in the list
            foreach (IDiagramItem idi in items)
            {
                idi.draw(g, showVision, showRepulsion, (count == currentSelection));
                count++;
            }
        }

        // returns a list of strings to be exported
        public List<List<string>> getExportData()
        {
            List<List<string>> l = new List<List<string>>();

            List<string> viruses = new List<string>();
            List<string> cells = new List<string>();

            // sets up the filename extension to use for each type of enemy
            viruses.Add("_viruses");
            cells.Add("_cells");

            // gets the export data from each item in the list
            foreach (IDiagramItem idi in items)
            {
                // puts each type of enemy in a different list
                switch (((CEnemyItem)idi).Type)
                {
                    case eEnemy.virus:
                        {
                            viruses.Add(idi.getExportData());
                            break;
                        }
                    case eEnemy.cell:
                        {
                            cells.Add(idi.getExportData());
                            break;
                        }
                    default:
                        {
                            throw new Exception("Unexpected case statement result.");
                        }
                }
            }

            // adds all the lists of strings to the list
            l.Add(viruses);
            l.Add(cells);

            return l;
        }

        // returns the current selection
        public CEnemy getSelection()
        {
            // prevents the index being out of range
            if (isValidSelection())
            {
                return (((CEnemy)items[currentSelection]));
            }
            else
            {
                return null;
            }
        }

        // checks to see if a selection is valid
        public bool isValidSelection()
        {
            return (currentSelection >= 0);
        }

        // opens a file containing data
        public static CDiagram load(string fileName)
        {
            try
            {
                // creates a new filestream object
                FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read);

                // creates a new binary formatter
                BinaryFormatter bf = new BinaryFormatter();

                CDiagram newDiagram;

                if (fs.Length > 0)
                {
                    newDiagram = (CDiagram)bf.Deserialize(fs);

                    // ensures nothing starts as selected
                    newDiagram.currentSelection = -1;
                }
                else
                {
                    throw new Exception("Error, cannot open empty file.");
                }

                // closes the filestream
                fs.Close();

                return newDiagram;
            }
            catch(Exception exc)
            {
                throw new Exception(exc.Message);
            }
        }

        // moves the current selection to a specified position 
        public void moveSelection(Point newPosition, bool usingMouse)
        {
            ((IDiagramItem)items[currentSelection]).updatePosition(newPosition, usingMouse);
        }

        // resets the diagram
        public void reset()
        {
            items.Clear();
            currentSelection = -1;
        }

        // saves the current data
        public void save(string fileName)
        {

            try
            {
                // creates a new filestream object
                FileStream fs = new FileStream(fileName, FileMode.OpenOrCreate);

                // creates a new binary formatter
                BinaryFormatter bf = new BinaryFormatter();

                bf.Serialize(fs, this);

                // closes the filestream
                fs.Close();
            }
            catch
            {
                throw new Exception("Error saving data.");
            }
        }

        // searches the list for an enemy that contains the specified point
        public bool select(Point searchLocation)
        {
            int count = 0;
            int previousCurrentSelection = currentSelection;

            // loops through every entity and finds the last drawing item
            // that contains the specified point
            foreach (IDiagramItem idi in items)
            {
                if (idi.containsPoint(searchLocation))
                {
                    currentSelection = count;
                }

                count++;
            }

            return (currentSelection != previousCurrentSelection);
        }

        #endregion
    }
}
