﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace EnemyPlacementTester
{
    [Serializable]
    public class CEnemy : CEnemyItem
    {
        public CEnemy(eEnemy newType, Image newImage, Point newPoint, int newVisionRange, int newRepulsionRange)
            : base(newType, newImage, newPoint, newVisionRange, newRepulsionRange)
        {
            // base constructor called
        }

        // returns whether a point is contained within this entity's drawing area
        public override bool containsPoint(Point p)
        {
            if (p.X < area.X)
            {
                return false;
            }

            if (p.X > area.X + area.Width)
            {
                return false;
            }

            if (p.Y < area.Y)
            {
                return false;
            }

            if (p.Y > area.Y + area.Height)
            {
                return false;
            }

            // if all other conditions fail then the point
            // is within the rectangle
            return true;
        }

        // draws the entity
        public override void draw(Graphics g, bool showVision, bool showRepulsion, bool currentSelection)
        {
            // draws the stored sprite at the specfied location
            g.DrawImage(sprite, area.X, area.Y, sprite.Width, sprite.Height);

            // displays vision range if required
            if (showVision)
            {
                drawRange(g, visionRange, Color.Blue);
            }

            // displays repulsion range if required
            if (showRepulsion)
            {
                drawRange(g, repulsionRange, Color.Red);
            }

            // draws a rectangle around the entity if it is currently selected
            if (currentSelection)
            {
                g.DrawRectangle(new Pen(Color.Blue), area);
            }
        }

        // draws a circle to represent a specified range around the centre of the entity
        private void drawRange(Graphics g, int range, Color colour)
        {
            // calculates the centre point of this rectangle
            Point centre = new Point(area.X + (area.Width / 2), area.Y + (area.Height / 2));
            Rectangle rangeRect = new Rectangle(centre.X - range, centre.Y - range, range * 2, range * 2);
            g.DrawEllipse(new Pen(colour), rangeRect);
        }

        // exports the data necessary
        public override string getExportData()
        {
            return(area.X.ToString() + "," + area.Y.ToString());
        }

        // updates the entity position and drawing rectangle
        public override void updatePosition(Point newPosition, bool usingMouse)
        {
            area.X = newPosition.X - (area.Width / 2);
            area.Y = newPosition.Y - (area.Height / 2);
        }
    }
}
